@@ -1,44 +1,54 @@
-import exceptions
 import json
 import pexpect
 import sys
 import tty
 
 
+# pexpect helper
 class Spawn(pexpect.spawn):
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('env', {})['TERM'] = 'dumb'
         super(Spawn, self).__init__(*args, **kwargs)
+        tty.setraw(self.child_fd)
 
-    def sendline_noecho(self, *args, **kwargs):
+    def noecho(self):
         self.setecho(False)
         self.waitnoecho()
-        return super(Spawn, self).sendline(*args, **kwargs)
+
+    def sendline_noecho(self, *args, **kwargs):
+        self.noecho()
+        return self.sendline(*args, **kwargs)
+
+    def expect_exact_noecho(self, *args, **kwargs):
+        self.noecho()
+        return self.expect_exact(*args, **kwargs)
 
 
-class BondException(exceptions.IOError):
+# Our exceptions
+class BondException(RuntimeError):
     def __init__(self, lang, error):
         self.lang = lang
+        self.error = error
         super(BondException, self).__init__(error)
 
     def __str__(self):
-        return "BondException[{lang}]: {msg}".format(lang=self.lang, msg=self.message)
+        return "BondException[{lang}]: {msg}".format(lang=self.lang, msg=self.error)
 
 class TerminatedException(BondException):
     def __init__(self, lang, error):
         super(TerminatedException, self).__init__(lang, error)
 
     def __str__(self):
-        return "TerminatedException[{lang}]: {msg}".format(lang=self.lang, msg=self.message)
+        return "TerminatedException[{lang}]: {msg}".format(lang=self.lang, msg=self.error)
 
-class SerializationException(BondException):
+class SerializationException(BondException, TypeError):
     def __init__(self, lang, error, side):
         self.side = side
         super(SerializationException, self).__init__(lang, error)
 
     def __str__(self):
         return "SerializationException[{lang}, {side}]: {msg}".format(
-            lang=self.lang, side=self.side, msg=self.message)
+            lang=self.lang, side=self.side, msg=self.error)
 
 class RemoteException(BondException):
     def __init__(self, lang, error, data):
@@ -46,30 +56,57 @@ class RemoteException(BondException):
         super(RemoteException, self).__init__(lang, error)
 
     def __str__(self):
-        return "RemoteException[{lang}]: {msg}".format(lang=self.lang, msg=self.message)
+        return "RemoteException[{lang}]: {msg}".format(lang=self.lang, msg=self.error)
 
 
+# The main base class
 class Bond(object):
     LANG = '<unknown>'
 
     def __init__(self, proc, trans_except):
         self.channels = {'STDOUT': sys.stdout, 'STDERR': sys.stderr}
         self.bindings = {}
-
         self.trans_except = trans_except
         self._proc = proc
+
+
+    def _init_2stage(self, proc, probe, stage1, stage2, trans_except):
+        # probe the interpreter
         try:
-            self._proc.expect_exact("READY\r\n")
+            proc.sendline_noecho(probe)
+            if proc.expect_exact_noecho(['STAGE1\n', 'STAGE1\r\n']) == 1:
+                tty.setraw(proc.child_fd)
         except pexpect.ExceptionPexpect:
-            raise BondException(self.LANG, 'unknown interpreter state')
-        tty.setraw(self._proc.child_fd)
+            raise BondException(self.LANG,
+                                'cannot get an interactive prompt using: '
+                                + str(proc.args))
+
+        # inject base loader
+        try:
+            proc.sendline(stage1)
+            if proc.expect_exact_noecho(['STAGE2\n', 'STAGE2\r\n']) == 1:
+                raise BondException(self.LANG, 'cannot switch terminal to raw mode')
+        except pexpect.ExceptionPexpect:
+            errors = proc.before.decode('utf-8')
+            raise BondException(self.LANG, 'cannot initialize stage1: ' + errors)
+
+        # load the second stage
+        try:
+            proc.sendline(stage2)
+            proc.expect_exact("READY\n")
+        except pexpect.ExceptionPexpect:
+            errors = proc.before.decode('utf-8')
+            raise BondException(self.LANG, 'cannot initialize stage2: ' + errors)
+
+        # remote environment is ready
+        Bond.__init__(self, proc, trans_except)
 
 
-    def loads(self, string):
-        return json.loads(string)
+    def loads(self, buf):
+        return json.loads(buf.decode('utf-8'))
 
     def dumps(self, *args):
-        return json.dumps(*args, skipkeys=False)
+        return json.dumps(*args, skipkeys=False).encode('utf-8')
 
 
     def _dumps(self, *args):
@@ -78,11 +115,15 @@ class Bond(object):
         except Exception as e:
             raise SerializationException(self.LANG, str(e), 'local')
 
+    def _sendstate(self, cmd, code):
+        ret = bytes(cmd.encode('ascii')) + b' ' + code
+        self._proc.sendline(ret)
+
 
     def _repl(self):
-        while self._proc.expect_exact('\n') == 0:
-            line = self._proc.before.split(' ', 1)
-            cmd = str(line[0])
+        while self._proc.expect_exact(b'\n') == 0:
+            line = self._proc.before.split(b' ', 1)
+            cmd = line[0].decode('ascii')
             args = self.loads(line[1]) if len(line) > 1 else []
 
             # interpret the serial protocol
@@ -110,7 +151,7 @@ class Bond(object):
                 except SerializationException as e:
                     state = "ERROR"
                     code = self._dumps(str(e))
-                self._proc.sendline('{state} {code}'.format(state=state, code=code))
+                self._sendstate(state, code)
                 continue
 
             raise BondException(self.LANG, 'unknown interpreter state')
@@ -118,20 +159,17 @@ class Bond(object):
 
     def eval(self, code):
         '''Evaluate and return the value of a single statement of code in the interpreter.'''
-        code = self._dumps(code)
-        self._proc.sendline('EVAL {code}'.format(code=code))
+        self._sendstate('EVAL', self._dumps(code))
         return self._repl()
 
     def eval_block(self, code):
         '''Evaluate a "code" block inside the interpreter. Nothing is returned.'''
-        code = self._dumps(code)
-        self._proc.sendline('EVAL_BLOCK {code}'.format(code=code))
+        self._sendstate('EVAL_BLOCK', self._dumps(code))
         return self._repl()
 
     def call(self, name, *args):
         '''Call a function "name" using *args (apply *args to a callable statement "name")'''
-        code = self._dumps([name, args])
-        self._proc.sendline('CALL {code}'.format(code=code))
+        self._sendstate('CALL', self._dumps([name, args]))
         return self._repl()
 
     def close(self):
@@ -143,8 +181,8 @@ class Bond(object):
         If "name" is not specified, use the local function name directly.'''
         if name is None:
             name = func.__name__
+        self._sendstate('EXPORT', self._dumps(name))
         self.bindings[name] = func
-        self._proc.sendline('EXPORT {name}'.format(name=self._dumps(name)))
         return self._repl()
 
     def callable(self, name):
@@ -162,6 +200,7 @@ class Bond(object):
 
 
 
+# Utilities
 def interact(bond, prompt=None):
     '''Start an interactive session with "bond"
 
