@@ -8,6 +8,11 @@ def test_basic():
     js.close()
 
 
+def test_basic_rmt():
+    js = JavaScript("ssh localhost nodejs", timeout=1)
+    js.close()
+
+
 def test_call_marshalling():
     js = JavaScript(timeout=1)
 
@@ -124,6 +129,40 @@ def test_eval():
     assert(js.eval('test_js(0)') == 1)
 
 
+def test_eval_sentinel():
+    js = JavaScript(timeout=1)
+
+    # ensure the sentinel is not accessible
+    failed = False
+    try:
+        js.eval('SENTINEL')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(True)
+
+
+def test_eval_rec():
+    js = JavaScript(timeout=1)
+
+    # in a recursive call, we should still be able to see our global scope
+    def call_me():
+        assert(js.eval('should_exist') == 1)
+
+        failed = False
+        try:
+            js.eval('should_not_exist')
+        except bond.RemoteException as e:
+            print(e)
+            failed = True
+        assert(failed)
+
+    js.export(call_me)
+    js.eval_block('var should_exist = 1;')
+    assert(js.eval('should_exist') == 1)
+    js.call('call_me')
+
+
 def test_eval_error():
     js = JavaScript(timeout=1)
 
@@ -499,10 +538,17 @@ def test_stack_depth():
     assert(bond_repl_depth(js) == 1)
 
 
-def test_buf_size():
-    js = JavaScript(timeout=1)
+def _test_buf_size(js):
     for size in [2 ** n for n in range(9, 16)]:
         print("testing buffer >= {} bytes".format(size))
         buf = "x" * size
         ret = js.call('String', buf)
         assert(ret == str(ret))
+
+def test_buf_size():
+    js = JavaScript(timeout=1)
+    _test_buf_size(js)
+
+def test_buf_size_rmt():
+    js = JavaScript("ssh localhost nodejs", timeout=1)
+    _test_buf_size(js)
