@@ -8,6 +8,11 @@ def test_basic():
     php.close()
 
 
+def test_basic_rmt():
+    php = PHP("ssh localhost php", timeout=1)
+    php.close()
+
+
 def test_call_marshalling():
     php = PHP(timeout=1)
 
@@ -159,6 +164,37 @@ def test_eval():
     php.eval_block('$x = 1;')
     assert(php.eval('$x') == 1)
 
+    # unset a variable to stress our simulated global scope
+    php.eval_block('unset($x);')
+    assert(php.eval('!isset($x)') == 1)
+
+
+def test_eval_sentinel():
+    php = PHP(timeout=1)
+
+    # ensure the sentinel is not accessible
+    failed = False
+    try:
+        php.eval('$SENTINEL')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(failed)
+
+
+def test_eval_rec():
+    php = PHP(timeout=1)
+
+    # in a recursive call, we should still be able to see our global scope
+    def call_me():
+        assert(php.eval('$should_exist') == 1)
+        assert(php.eval('!isset($should_not_exist)'))
+
+    php.export(call_me)
+    php.eval_block('$should_exist = 1;')
+    assert(php.eval('$should_exist') == 1)
+    php.call('call_me')
+
 
 def test_eval_error():
     php = PHP(timeout=1)
@@ -493,8 +529,7 @@ def test_stack_depth():
     assert(bond_repl_depth(php) == 1)
 
 
-def test_buf_size():
-    php = PHP(timeout=1)
+def _test_buf_size(php):
     php.eval_block('function id($arg) { return $arg; }')
 
     for size in [2 ** n for n in range(9, 16)]:
@@ -502,3 +537,11 @@ def test_buf_size():
         buf = "x" * size
         ret = php.call('id', buf)
         assert(ret == str(ret))
+
+def test_buf_size():
+    php = PHP(timeout=1)
+    _test_buf_size(php)
+
+def test_buf_size_rmt():
+    php = PHP("ssh localhost php", timeout=1)
+    _test_buf_size(php)
