@@ -8,6 +8,11 @@ def test_basic():
     perl.close()
 
 
+def test_basic_rmt():
+    perl = Perl("ssh localhost perl", timeout=1)
+    perl.close()
+
+
 def test_call_marshalling():
     perl = Perl(timeout=1)
 
@@ -162,11 +167,42 @@ def test_eval():
     perl.eval_block('my $y = 1;')
     assert(perl.eval('$y // -1') == -1)
 
+    # 'our' scope
+    perl.eval_block('our $z = 1;')
+    assert(perl.eval('$z // -1') == 1)
+
     # function definition
     perl.eval('sub test_perl { shift() + 1; }')
     assert(perl.eval('test_perl(0);') == 1)
 
 
+def test_eval_sentinel():
+    perl = Perl(timeout=1)
+
+    # ensure the sentinel is not accessible
+    failed = False
+    try:
+        perl.eval('$SENTINEL')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(True)
+
+
+def test_eval_rec():
+    perl = Perl(timeout=1)
+
+    # in a recursive call, we should still be able to see our global scope
+    def call_me():
+        assert(perl.eval('$should_exist') == 1)
+        assert(perl.eval('$should_not_exist // -1') == -1)
+
+    perl.export(call_me)
+    perl.eval_block('$should_exist = 1')
+    assert(perl.eval('$should_exist') == 1)
+    perl.call('call_me')
+
+
 def test_ser_err():
     perl = Perl(timeout=1, trans_except=True)
 
@@ -511,12 +547,18 @@ def test_stack_depth():
     assert(bond_repl_depth(perl) == 1)
 
 
-def test_buf_size():
-    perl = Perl(timeout=1)
+def _test_buf_size(perl):
     perl.eval_block('sub id { shift() }')
-
     for size in [2 ** n for n in range(9, 16)]:
         print("testing buffer >= {} bytes".format(size))
         buf = "x" * size
         ret = perl.call('id', buf)
         assert(ret == str(ret))
+
+def test_buf_size():
+    perl = Perl(timeout=1)
+    _test_buf_size(perl)
+
+def test_buf_size_rmt():
+    perl = Perl("ssh localhost perl", timeout=1)
+    _test_buf_size(perl)
