@@ -8,6 +8,11 @@ def test_basic():
     py.close()
 
 
+def test_basic_rmt():
+    py = Python("ssh localhost python", timeout=1)
+    py.close()
+
+
 def test_call_marshalling():
     py = Python(timeout=1)
 
@@ -134,6 +139,40 @@ def test_eval():
     assert(py.eval('test_python(0)') == 1)
 
 
+def test_eval_sentinel():
+    py = Python(timeout=1)
+
+    # ensure the sentinel is not accessible
+    failed = False
+    try:
+        py.eval('SENTINEL')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(True)
+
+
+def test_eval_rec():
+    py = Python(timeout=1)
+
+    # in a recursive call, we should still be able to see our global scope
+    def call_me():
+        assert(py.eval('should_exist') == 1)
+
+        failed = False
+        try:
+            py.eval('should_not_exist')
+        except bond.RemoteException as e:
+            print(e)
+            failed = True
+        assert(failed)
+
+    py.export(call_me)
+    py.eval_block('should_exist = 1')
+    assert(py.eval('should_exist') == 1)
+    py.call('call_me')
+
+
 def test_eval_error():
     py = Python(timeout=1)
 
@@ -353,6 +392,7 @@ def test_export_except():
     try:
         gen_exception()
     except Exception as e:
+        print(e)
         ret = (str(e) == "test")
     return ret
     ''')
@@ -382,13 +422,14 @@ def test_export_except_ser_err():
 
 def test_output_redirect():
     py = Python(timeout=1)
+    py.eval_block(r'import sys')
 
     # stdout
-    py.eval_block(r'print "stdout: Hello world!"')
+    py.eval_block(r'sys.stdout.write("stdout: Hello world!\n")')
     assert(py.eval('1') == 1)
 
     # stderr
-    py.eval_block(r'import sys; sys.stderr.write("stderr: Hello world!\n");')
+    py.eval_block(r'sys.stderr.write("stderr: Hello world!\n")')
     assert(py.eval('1') == 1)
 
 
@@ -503,10 +544,17 @@ def test_stack_depth():
     assert(bond_repl_depth(py) == 1)
 
 
-def test_buf_size():
-    py = Python(timeout=1)
+def _test_buf_size(py):
     for size in [2 ** n for n in range(9, 16)]:
         print("testing buffer >= {} bytes".format(size))
         buf = "x" * size
         ret = py.call('str', buf)
         assert(ret == str(ret))
+
+def test_buf_size():
+    py = Python(timeout=1)
+    _test_buf_size(py)
+
+def test_buf_size_rmt():
+    py = Python("ssh localhost python", timeout=1)
+    _test_buf_size(py)
